@@ -29,6 +29,7 @@ modules = [
     "bbc",
     "behance",
     "blogger",
+    "bluesky",
     "bunkr",
     "catbox",
     "chevereto",
@@ -61,6 +62,7 @@ modules = [
     "hentaifox",
     "hentaihand",
     "hentaihere",
+    "hentainexus",
     "hiperdex",
     "hitomi",
     "hotleak",
@@ -112,7 +114,6 @@ modules = [
     "paheal",
     "patreon",
     "philomena",
-    "photobucket",
     "photovogue",
     "picarto",
     "piczel",
@@ -193,7 +194,6 @@ modules = [
     "directlink",
     "recursive",
     "oauth",
-    "test",
     "ytdl",
     "generic",
 ]
