@@ -11,9 +11,11 @@
 import os
 import re
 import ssl
+import sys
 import time
 import netrc
 import queue
+import getpass
 import logging
 import datetime
 import requests
@@ -21,6 +23,7 @@ import threading
 from requests.adapters import HTTPAdapter
 from .message import Message
 from .. import config, text, util, cache, exception
+urllib3 = requests.packages.urllib3
 
 
 class Extractor():
@@ -40,11 +43,14 @@ class Extractor():
     browser = None
     request_interval = 0.0
     request_interval_min = 0.0
+    request_interval_429 = 60.0
     request_timestamp = 0.0
 
     def __init__(self, match):
         self.log = logging.getLogger(self.category)
         self.url = match.string
+        self.match = match
+        self.groups = match.groups()
         self._cfgpath = ("extractor", self.category, self.subcategory)
         self._parentdir = ""
 
@@ -168,22 +174,25 @@ class Extractor():
                     requests.exceptions.ChunkedEncodingError,
                     requests.exceptions.ContentDecodingError) as exc:
                 msg = exc
+                code = 0
             except (requests.exceptions.RequestException) as exc:
                 raise exception.HttpError(exc)
             else:
                 code = response.status_code
                 if self._write_pages:
                     self._dump_response(response)
-                if 200 <= code < 400 or fatal is None and \
-                        (400 <= code < 500) or not fatal and \
-                        (400 <= code < 429 or 431 <= code < 500):
+                if (
+                    code < 400 or
+                    code < 500 and (not fatal and code != 429 or fatal is None)
+                ):
                     if encoding:
                         response.encoding = encoding
                     return response
                 if notfound and code == 404:
                     raise exception.NotFoundError(notfound)
 
-                msg = "'{} {}' for '{}'".format(code, response.reason, url)
+                msg = "'{} {}' for '{}'".format(
+                    code, response.reason, response.url)
                 server = response.headers.get("Server")
                 if server and server.startswith("cloudflare") and \
                         code in (403, 503):
@@ -194,7 +203,12 @@ class Extractor():
                     if b'name="captcha-bypass"' in content:
                         self.log.warning("Cloudflare CAPTCHA")
                         break
-                if code not in retry_codes and code < 500:
+
+                if code == 429 and self._handle_429(response):
+                    continue
+                elif code == 429 and self._interval_429:
+                    pass
+                elif code not in retry_codes and code < 500:
                     break
 
             finally:
@@ -203,15 +217,27 @@ class Extractor():
             self.log.debug("%s (%s/%s)", msg, tries, retries+1)
             if tries > retries:
                 break
-            self.sleep(
-                max(tries, self._interval()) if self._interval else tries,
-                "retry")
+
+            seconds = tries
+            if self._interval:
+                s = self._interval()
+                if seconds < s:
+                    seconds = s
+            if code == 429 and self._interval_429:
+                s = self._interval_429()
+                if seconds < s:
+                    seconds = s
+                self.wait(seconds=seconds, reason="429 Too Many Requests")
+            else:
+                self.sleep(seconds, "retry")
             tries += 1
 
         raise exception.HttpError(msg, response)
 
+    _handle_429 = util.false
+
     def wait(self, seconds=None, until=None, adjust=1.0,
-             reason="rate limit reset"):
+             reason="rate limit"):
         now = time.time()
 
         if seconds:
@@ -234,7 +260,7 @@ class Extractor():
         if reason:
             t = datetime.datetime.fromtimestamp(until).time()
             isotime = "{:02}:{:02}:{:02}".format(t.hour, t.minute, t.second)
-            self.log.info("Waiting until %s for %s.", isotime, reason)
+            self.log.info("Waiting until %s (%s)", isotime, reason)
         time.sleep(seconds)
 
     def sleep(self, seconds, reason):
@@ -242,13 +268,41 @@ class Extractor():
                        seconds, reason)
         time.sleep(seconds)
 
+    def input(self, prompt, echo=True):
+        self._check_input_allowed(prompt)
+
+        if echo:
+            try:
+                return input(prompt)
+            except (EOFError, OSError):
+                return None
+        else:
+            return getpass.getpass(prompt)
+
+    def _check_input_allowed(self, prompt=""):
+        input = self.config("input")
+
+        if input is None:
+            try:
+                input = sys.stdin.isatty()
+            except Exception:
+                input = False
+
+        if not input:
+            raise exception.StopExtraction(
+                "User input required (%s)", prompt.strip(" :"))
+
     def _get_auth_info(self):
         """Return authentication information as (username, password) tuple"""
         username = self.config("username")
         password = None
 
         if username:
-            password = self.config("password") or util.LazyPrompt()
+            password = self.config("password")
+            if not password:
+                self._check_input_allowed("password")
+                password = util.LazyPrompt()
+
         elif self.config("netrc", False):
             try:
                 info = netrc.netrc().authenticators(self.category)
@@ -274,6 +328,9 @@ class Extractor():
             self.config("sleep-request", self.request_interval),
             self.request_interval_min,
         )
+        self._interval_429 = util.build_duration_func(
+            self.config("sleep-429", self.request_interval_429),
+        )
 
         if self._retries < 0:
             self._retries = float("inf")
@@ -433,9 +490,11 @@ class Extractor():
             if not path:
                 return
 
+        path_tmp = path + ".tmp"
         try:
-            with open(path, "w") as fp:
+            with open(path_tmp, "w") as fp:
                 util.cookiestxt_store(fp, self.cookies)
+            os.replace(path_tmp, path)
         except OSError as exc:
             self.log.warning("cookies: %s", exc)
 
@@ -593,7 +652,7 @@ class GalleryExtractor(Extractor):
 
     def __init__(self, match, url=None):
         Extractor.__init__(self, match)
-        self.gallery_url = self.root + match.group(1) if url is None else url
+        self.gallery_url = self.root + self.groups[0] if url is None else url
 
     def items(self):
         self.login()
@@ -668,7 +727,7 @@ class MangaExtractor(Extractor):
 
     def __init__(self, match, url=None):
         Extractor.__init__(self, match)
-        self.manga_url = url or self.root + match.group(1)
+        self.manga_url = self.root + self.groups[0] if url is None else url
 
         if self.config("chapter-reverse", False):
             self.reverse = not self.reverse
@@ -730,17 +789,18 @@ class BaseExtractor(Extractor):
     instances = ()
 
     def __init__(self, match):
-        if not self.category:
-            self._init_category(match)
         Extractor.__init__(self, match)
+        if not self.category:
+            self._init_category()
+            self._cfgpath = ("extractor", self.category, self.subcategory)
 
-    def _init_category(self, match):
-        for index, group in enumerate(match.groups()):
+    def _init_category(self):
+        for index, group in enumerate(self.groups):
             if group is not None:
                 if index:
                     self.category, self.root, info = self.instances[index-1]
                     if not self.root:
-                        self.root = text.root_from_url(match.group(0))
+                        self.root = text.root_from_url(self.match.group(0))
                     self.config_instance = info.get
                 else:
                     self.root = group
@@ -800,12 +860,12 @@ def _build_requests_adapter(ssl_options, ssl_ciphers, source_address):
         pass
 
     if ssl_options or ssl_ciphers:
-        ssl_context = ssl.create_default_context()
-        if ssl_options:
-            ssl_context.options |= ssl_options
-        if ssl_ciphers:
-            ssl_context.set_ecdh_curve("prime256v1")
-            ssl_context.set_ciphers(ssl_ciphers)
+        ssl_context = urllib3.connection.create_urllib3_context(
+            options=ssl_options or None, ciphers=ssl_ciphers)
+        if not requests.__version__ < "2.32":
+            # https://github.com/psf/requests/pull/6731
+            ssl_context.load_default_certs()
+        ssl_context.check_hostname = False
     else:
         ssl_context = None
 
@@ -925,8 +985,6 @@ SSL_CIPHERS = {
 }
 
 
-urllib3 = requests.packages.urllib3
-
 # detect brotli support
 try:
     BROTLI = urllib3.response.brotli is not None
