@@ -1,26 +1,37 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2015-2022 Mike Fährmann
+# Copyright 2015-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation.
 
 import sys
-import json
-import time
 import errno
 import logging
 import functools
 import collections
-from . import extractor, downloader, postprocessor
-from . import config, text, util, path, formatter, output, exception
+
+from . import (
+    extractor,
+    downloader,
+    postprocessor,
+    archive,
+    config,
+    exception,
+    formatter,
+    output,
+    path,
+    text,
+    util,
+    version,
+)
 from .extractor.message import Message
-from .output import stdout_write
+stdout_write = output.stdout_write
 
 
 class Job():
-    """Base class for Job-types"""
+    """Base class for Job types"""
     ulog = None
 
     def __init__(self, extr, parent=None):
@@ -31,13 +42,46 @@ class Job():
 
         self.extractor = extr
         self.pathfmt = None
-        self.kwdict = {}
         self.status = 0
-        self.url_key = extr.config("url-metadata")
+        self.kwdict = {}
+        self.kwdict_eval = False
+
+        cfgpath = []
+        if parent:
+            if extr.category == parent.extractor.category or \
+                    extr.category in parent.parents:
+                parents = parent.parents
+            else:
+                parents = parent.parents + (parent.extractor.category,)
+
+            if parents:
+                for category in parents:
+                    cat = "{}>{}".format(category, extr.category)
+                    cfgpath.append((cat, extr.subcategory))
+                cfgpath.append((extr.category, extr.subcategory))
+                self.parents = parents
+            else:
+                self.parents = ()
+        else:
+            self.parents = ()
+
+        if extr.basecategory:
+            if not cfgpath:
+                cfgpath.append((extr.category, extr.subcategory))
+            cfgpath.append((extr.basecategory, extr.subcategory))
+
+        if cfgpath:
+            extr._cfgpath = cfgpath
+            extr.config = extr._config_shared
+            extr.config_accumulate = extr._config_shared_accumulate
+
+        actions = extr.config("actions")
+        if actions:
+            from .actions import parse
+            self._logger_actions = parse(actions)
+            self._wrap_logger = self._wrap_logger_actions
 
-        path_key = extr.config("path-metadata")
         path_proxy = output.PathfmtProxy(self)
-
         self._logger_extra = {
             "job"      : self,
             "extractor": extr,
@@ -57,16 +101,36 @@ class Job():
                 extr.category = pextr.category
                 extr.subcategory = pextr.subcategory
 
+        self.metadata_url = extr.config2("metadata-url", "url-metadata")
+        self.metadata_http = extr.config2("metadata-http", "http-metadata")
+        metadata_path = extr.config2("metadata-path", "path-metadata")
+        metadata_version = extr.config2("metadata-version", "version-metadata")
+        metadata_extractor = extr.config2(
+            "metadata-extractor", "extractor-metadata")
+
+        if metadata_path:
+            self.kwdict[metadata_path] = path_proxy
+        if metadata_extractor:
+            self.kwdict[metadata_extractor] = extr
+        if metadata_version:
+            self.kwdict[metadata_version] = {
+                "version"         : version.__version__,
+                "is_executable"   : util.EXECUTABLE,
+                "current_git_head": util.git_head()
+            }
         # user-supplied metadata
         kwdict = extr.config("keywords")
         if kwdict:
-            self.kwdict.update(kwdict)
-        if path_key:
-            self.kwdict[path_key] = path_proxy
-
-        # predicates
-        self.pred_url = self._prepare_predicates("image", True)
-        self.pred_queue = self._prepare_predicates("chapter", False)
+            if extr.config("keywords-eval"):
+                self.kwdict_eval = []
+                for key, value in kwdict.items():
+                    if isinstance(value, str):
+                        fmt = formatter.parse(value, None, util.identity)
+                        self.kwdict_eval.append((key, fmt.format_map))
+                    else:
+                        self.kwdict[key] = value
+            else:
+                self.kwdict.update(kwdict)
 
     def run(self):
         """Execute or run the job"""
@@ -74,9 +138,13 @@ class Job():
         log = extractor.log
         msg = None
 
-        sleep = util.build_duration_func(extractor.config("sleep-extractor"))
+        self._init()
+
+        # sleep before extractor start
+        sleep = util.build_duration_func(
+            extractor.config("sleep-extractor"))
         if sleep:
-            time.sleep(sleep())
+            extractor.sleep(sleep(), "extractor")
 
         try:
             for msg in extractor:
@@ -85,7 +153,7 @@ class Job():
             if exc.message:
                 log.error(exc.message)
             self.status |= exc.code
-        except exception.TerminateExtraction:
+        except (exception.TerminateExtraction, exception.RestartExtraction):
             raise
         except exception.GalleryDLException as exc:
             log.error("%s: %s", exc.__class__.__name__, exc)
@@ -111,8 +179,7 @@ class Job():
                 log.info("No results for %s", extractor.url)
         finally:
             self.handle_finalize()
-            if extractor.finalize:
-                extractor.finalize()
+            extractor.finalize()
 
         return self.status
 
@@ -120,8 +187,8 @@ class Job():
         """Call the appropriate message handler"""
         if msg[0] == Message.Url:
             _, url, kwdict = msg
-            if self.url_key:
-                kwdict[self.url_key] = url
+            if self.metadata_url:
+                kwdict[self.metadata_url] = url
             if self.pred_url(url, kwdict):
                 self.update_kwdict(kwdict)
                 self.handle_url(url, kwdict)
@@ -132,8 +199,8 @@ class Job():
 
         elif msg[0] == Message.Queue:
             _, url, kwdict = msg
-            if self.url_key:
-                kwdict[self.url_key] = url
+            if self.metadata_url:
+                kwdict[self.metadata_url] = url
             if self.pred_queue(url, kwdict):
                 self.handle_queue(url, kwdict)
 
@@ -154,8 +221,18 @@ class Job():
         extr = self.extractor
         kwdict["category"] = extr.category
         kwdict["subcategory"] = extr.subcategory
+        if self.metadata_http:
+            kwdict.pop(self.metadata_http, None)
         if self.kwdict:
             kwdict.update(self.kwdict)
+        if self.kwdict_eval:
+            for key, valuegen in self.kwdict_eval:
+                kwdict[key] = valuegen(kwdict)
+
+    def _init(self):
+        self.extractor.initialize()
+        self.pred_url = self._prepare_predicates("image", True)
+        self.pred_queue = self._prepare_predicates("chapter", False)
 
     def _prepare_predicates(self, target, skip=True):
         predicates = []
@@ -190,7 +267,10 @@ class Job():
         return self._wrap_logger(logging.getLogger(name))
 
     def _wrap_logger(self, logger):
-        return output.LoggerAdapter(logger, self._logger_extra)
+        return output.LoggerAdapter(logger, self)
+
+    def _wrap_logger_actions(self, logger):
+        return output.LoggerAdapterActions(logger, self)
 
     def _write_unsupported(self, url):
         if self.ulog:
@@ -231,14 +311,21 @@ class DownloadJob(Job):
             self.handle_skip()
             return
 
-        if pathfmt.exists():
-            if archive:
-                archive.add(kwdict)
-            self.handle_skip()
-            return
+        if pathfmt.extension and not self.metadata_http:
+            pathfmt.build_path()
+
+            if pathfmt.exists():
+                if archive:
+                    archive.add(kwdict)
+                self.handle_skip()
+                return
+
+        if "prepare-after" in hooks:
+            for callback in hooks["prepare-after"]:
+                callback(pathfmt)
 
         if self.sleep:
-            time.sleep(self.sleep())
+            self.extractor.sleep(self.sleep(), "download")
 
         # download from URL
         if not self.download(url):
@@ -283,6 +370,9 @@ class DownloadJob(Job):
         if not self.pathfmt:
             self.initialize(kwdict)
         else:
+            if "post-after" in self.hooks:
+                for callback in self.hooks["post-after"]:
+                    callback(self.pathfmt)
             self.pathfmt.set_directory(kwdict)
         if "post" in self.hooks:
             for callback in self.hooks["post"]:
@@ -314,7 +404,7 @@ class DownloadJob(Job):
             else:
                 extr._parentdir = pextr._parentdir
 
-            pmeta = pextr.config("parent-metadata")
+            pmeta = pextr.config2("parent-metadata", "metadata-parent")
             if pmeta:
                 if isinstance(pmeta, str):
                     data = self.kwdict.copy()
@@ -327,25 +417,62 @@ class DownloadJob(Job):
                     if kwdict:
                         job.kwdict.update(kwdict)
 
-            if pextr.config("parent-skip"):
-                job._skipcnt = self._skipcnt
-                self.status |= job.run()
-                self._skipcnt = job._skipcnt
-            else:
-                self.status |= job.run()
+            while True:
+                try:
+                    if pextr.config("parent-skip"):
+                        job._skipcnt = self._skipcnt
+                        status = job.run()
+                        self._skipcnt = job._skipcnt
+                    else:
+                        status = job.run()
+
+                    if status:
+                        self.status |= status
+                        if "_fallback" in kwdict and self.fallback:
+                            fallback = kwdict["_fallback"] = \
+                                iter(kwdict["_fallback"])
+                            try:
+                                url = next(fallback)
+                            except StopIteration:
+                                pass
+                            else:
+                                text.nameext_from_url(url, kwdict)
+                                if url.startswith("ytdl:"):
+                                    kwdict["extension"] = ""
+                                self.handle_url(url, kwdict)
+                    break
+                except exception.RestartExtraction:
+                    pass
+
         else:
             self._write_unsupported(url)
 
     def handle_finalize(self):
-        pathfmt = self.pathfmt
         if self.archive:
+            if not self.status:
+                self.archive.finalize()
             self.archive.close()
+
+        pathfmt = self.pathfmt
         if pathfmt:
-            self.extractor._store_cookies()
-            if "finalize" in self.hooks:
-                status = self.status
-                for callback in self.hooks["finalize"]:
-                    callback(pathfmt, status)
+            hooks = self.hooks
+            if "post-after" in hooks:
+                for callback in hooks["post-after"]:
+                    callback(pathfmt)
+
+            self.extractor.cookies_store()
+
+            if "finalize" in hooks:
+                for callback in hooks["finalize"]:
+                    callback(pathfmt)
+            if self.status:
+                if "finalize-error" in hooks:
+                    for callback in hooks["finalize-error"]:
+                        callback(pathfmt)
+            else:
+                if "finalize-success" in hooks:
+                    for callback in hooks["finalize-success"]:
+                        callback(pathfmt)
 
     def handle_skip(self):
         pathfmt = self.pathfmt
@@ -354,9 +481,12 @@ class DownloadJob(Job):
             for callback in self.hooks["skip"]:
                 callback(pathfmt)
         if self._skipexc:
-            self._skipcnt += 1
-            if self._skipcnt >= self._skipmax:
-                raise self._skipexc()
+            if not self._skipftr or self._skipftr(pathfmt.kwdict):
+                self._skipcnt += 1
+                if self._skipcnt >= self._skipmax:
+                    raise self._skipexc()
+            else:
+                self._skipcnt = 0
 
     def download(self, url):
         """Download 'url'"""
@@ -408,21 +538,28 @@ class DownloadJob(Job):
             # monkey-patch method to do nothing and always return True
             self.download = pathfmt.fix_extension
 
-        archive = cfg("archive")
-        if archive:
-            archive = util.expand_path(archive)
+        archive_path = cfg("archive")
+        if archive_path:
+            archive_path = util.expand_path(archive_path)
             archive_format = (cfg("archive-prefix", extr.category) +
                               cfg("archive-format", extr.archive_fmt))
+            archive_pragma = (cfg("archive-pragma"))
             try:
-                if "{" in archive:
-                    archive = formatter.parse(archive).format_map(kwdict)
-                self.archive = util.DownloadArchive(archive, archive_format)
+                if "{" in archive_path:
+                    archive_path = formatter.parse(
+                        archive_path).format_map(kwdict)
+                if cfg("archive-mode") == "memory":
+                    archive_cls = archive.DownloadArchiveMemory
+                else:
+                    archive_cls = archive.DownloadArchive
+                self.archive = archive_cls(
+                    archive_path, archive_format, archive_pragma)
             except Exception as exc:
                 extr.log.warning(
-                    "Failed to open download archive at '%s' ('%s: %s')",
-                    archive, exc.__class__.__name__, exc)
+                    "Failed to open download archive at '%s' (%s: %s)",
+                    archive_path, exc.__class__.__name__, exc)
             else:
-                extr.log.debug("Using download archive '%s'", archive)
+                extr.log.debug("Using download archive '%s'", archive_path)
 
         skip = cfg("skip", True)
         if skip:
@@ -436,8 +573,14 @@ class DownloadJob(Job):
                 elif skip == "terminate":
                     self._skipexc = exception.TerminateExtraction
                 elif skip == "exit":
-                    self._skipexc = sys.exit
+                    self._skipexc = SystemExit
                 self._skipmax = text.parse_int(smax)
+
+                skip_filter = cfg("skip-filter")
+                if skip_filter:
+                    self._skipftr = util.compile_expression(skip_filter)
+                else:
+                    self._skipftr = None
         else:
             # monkey-patch methods to always return False
             pathfmt.exists = lambda x=None: False
@@ -450,13 +593,18 @@ class DownloadJob(Job):
         postprocessors = extr.config_accumulate("postprocessors")
         if postprocessors:
             self.hooks = collections.defaultdict(list)
+
             pp_log = self.get_logger("postprocessor")
+            pp_conf = config.get((), "postprocessor") or {}
+            pp_opts = cfg("postprocessor-options")
             pp_list = []
 
-            pp_conf = config.get((), "postprocessor") or {}
             for pp_dict in postprocessors:
                 if isinstance(pp_dict, str):
                     pp_dict = pp_conf.get(pp_dict) or {"name": pp_dict}
+                if pp_opts:
+                    pp_dict = pp_dict.copy()
+                    pp_dict.update(pp_opts)
 
                 clist = pp_dict.get("whitelist")
                 if clist is not None:
@@ -526,12 +674,11 @@ class SimulationJob(DownloadJob):
     def handle_url(self, url, kwdict):
         if not kwdict["extension"]:
             kwdict["extension"] = "jpg"
-        self.pathfmt.set_filename(kwdict)
-        self.out.skip(self.pathfmt.path)
         if self.sleep:
-            time.sleep(self.sleep())
+            self.extractor.sleep(self.sleep(), "download")
         if self.archive:
             self.archive.add(kwdict)
+        self.out.skip(self.pathfmt.build_filename(kwdict))
 
     def handle_directory(self, kwdict):
         if not self.pathfmt:
@@ -548,6 +695,11 @@ class KeywordJob(Job):
     def handle_url(self, url, kwdict):
         stdout_write("\nKeywords for filenames and --filter:\n"
                      "------------------------------------\n")
+
+        if self.metadata_http and url.startswith("http"):
+            kwdict[self.metadata_http] = util.extract_headers(
+                self.extractor.request(url, method="HEAD"))
+
         self.print_kwdict(kwdict)
         raise exception.StopExtraction()
 
@@ -585,13 +737,13 @@ class KeywordJob(Job):
     def print_kwdict(self, kwdict, prefix="", markers=None):
         """Print key-value pairs in 'kwdict' with formatting"""
         write = sys.stdout.write
-        suffix = "]" if prefix else ""
+        suffix = "']" if prefix else ""
 
         markerid = id(kwdict)
         if markers is None:
             markers = {markerid}
         elif markerid in markers:
-            write("{}\n  <circular reference>\n".format(prefix[:-1]))
+            write("{}\n  <circular reference>\n".format(prefix[:-2]))
             return  # ignore circular reference
         else:
             markers.add(markerid)
@@ -602,20 +754,25 @@ class KeywordJob(Job):
             key = prefix + key + suffix
 
             if isinstance(value, dict):
-                self.print_kwdict(value, key + "[", markers)
+                self.print_kwdict(value, key + "['", markers)
 
             elif isinstance(value, list):
-                if value and isinstance(value[0], dict):
-                    self.print_kwdict(value[0], key + "[][", markers)
+                if not value:
+                    pass
+                elif isinstance(value[0], dict):
+                    self.print_kwdict(value[0], key + "[N]['", markers)
                 else:
-                    write(key + "[]\n")
-                    for val in value:
-                        write("  - " + str(val) + "\n")
+                    fmt = ("  {:>%s} {}\n" % len(str(len(value)))).format
+                    write(key + "[N]\n")
+                    for idx, val in enumerate(value, 0):
+                        write(fmt(idx, val))
 
             else:
                 # string or number
                 write("{}\n  {}\n".format(key, value))
 
+        markers.remove(markerid)
+
 
 class UrlJob(Job):
     """Print download urls"""
@@ -674,17 +831,19 @@ class InfoJob(Job):
 
     def _print_multi(self, title, *values):
         stdout_write("{}\n  {}\n\n".format(
-            title, " / ".join(json.dumps(v) for v in values)))
+            title, " / ".join(map(util.json_dumps, values))))
 
     def _print_config(self, title, optname, value):
         optval = self.extractor.config(optname, util.SENTINEL)
         if optval is not util.SENTINEL:
             stdout_write(
                 "{} (custom):\n  {}\n{} (default):\n  {}\n\n".format(
-                    title, json.dumps(optval), title, json.dumps(value)))
+                    title, util.json_dumps(optval),
+                    title, util.json_dumps(value)))
         elif value:
             stdout_write(
-                "{} (default):\n  {}\n\n".format(title, json.dumps(value)))
+                "{} (default):\n  {}\n\n".format(
+                    title, util.json_dumps(value)))
 
 
 class DataJob(Job):
@@ -697,17 +856,20 @@ class DataJob(Job):
         self.ascii = config.get(("output",), "ascii", ensure_ascii)
 
         private = config.get(("output",), "private")
-        self.filter = util.identity if private else util.filter_dict
+        self.filter = dict.copy if private else util.filter_dict
 
     def run(self):
+        self._init()
+
+        extractor = self.extractor
         sleep = util.build_duration_func(
-            self.extractor.config("sleep-extractor"))
+            extractor.config("sleep-extractor"))
         if sleep:
-            time.sleep(sleep())
+            extractor.sleep(sleep(), "extractor")
 
         # collect data
         try:
-            for msg in self.extractor:
+            for msg in extractor:
                 self.dispatch(msg)
         except exception.StopExtraction:
             pass
